import { ModuleConfig } from './ModuleConfigure';
import { Registration, Token, IContainer } from '@ts-ioc/core';
/**
 * module metadata accessor
 *
 * @export
 * @interface IMetaAccessor
 * @template T
 */
export interface IMetaAccessor<T> {
    /**
     * get metadata of target type.
     *
     * @param {IContainer} container
     * @param {Token<T>} type
     * @returns {ModuleConfig<T>}
     * @memberof IMetaAccessor
     */
    get(container: IContainer, type: Token<T>): ModuleConfig<T>;
}
/**
 * application service token.
 *
 * @export
 * @class InjectMetaAccessorToken
 * @extends {Registration<MetaAccessor<T>>}
 * @template T
 */
export declare class InjectMetaAccessorToken<T> extends Registration<IMetaAccessor<T>> {
    constructor(type: Token<T>);
    static classAnnations: any;
}
/**
 * default MetaAccessor token.
 */
export declare const DefaultMetaAccessorToken: InjectMetaAccessorToken<any>;
export declare class MetaAccessor implements IMetaAccessor<any> {
    private decorator;
    constructor(decorator: string);
    get(container: IContainer, token: Token<any>): ModuleConfig<any>;
    static classAnnations: any;
}
